/**
 * Wackyd environment initialization.
 *
 * Copyright (C) 2001-2002 Jeffrey Fulmer <jdfulmer@armstrong.com>
 * This file is part of Siege
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */  
#include <init.h>
#include <setup.h>
#include <util.h>
char seti[128];

int
init_config( void )
{
  char filename[256];

  memset( &my, 0, sizeof( struct CONFIG )); 
  my.done     = FALSE;
  my.debug    = FALSE;
  my.config   = FALSE;
  my.speed    = 3;
  my.hostname = "";
  
  snprintf(
    filename, sizeof( filename),
    "%s/.wackyrc", getenv( "HOME" )
  );
 
  if( load_conf( filename ) < 0 ){
    fprintf( stderr, "****************************************************\n" );
    fprintf( stderr, "wackyd: could not open %s\n", filename );
    fprintf( stderr, "run \'wacky.config\' to generate a new .wackyrc file\n" );
    fprintf( stderr, "****************************************************\n" );
    return -1;
  }
  return 0;
}

int
show_config( int EXIT )
{
  printf( "CURRENT  WACKYD  CONFIGURATION\n" );
  printf( "Edit the resource file to change the settings.\n" );
  printf( "----------------------------------------------\n" );
  printf( "version:               %s\n", version_string );
  printf( "hostname:              %s\n", (my.hostname == NULL)?"*":my.hostname );
  printf( "default port:          %s\n", (my.portname == NULL)?"9193":my.portname );
  printf( "default timeout:       %d\n", my.timeout?my.timeout:15 );
  printf( "default speed:         %d milliseconds\n", my.speed?my.speed:3 ); 
  printf( "resource file:         %s/.wackyrc\n", getenv( "HOME" )); 
  display_list();
  printf( "\n" );

  if( EXIT ) exit( 0 );
  else return 0;
}

static char
*get_line( FILE *fp )
{
  char *ptr;
  char *new;
  char tmp[256];
 
  ptr = xmalloc( 1 );
 
  do{
    if(( fgets( tmp, sizeof( tmp ), fp )) == NULL ) return( NULL );
    if( ptr == NULL ) ptr = strdup( tmp );
    else{
      if(( ptr = xrealloc( ptr, strlen(ptr) + strlen(tmp) + 1 )) == NULL ) break;
      strcat( ptr, tmp );
    }
  } while(( new = strchr( ptr, '\n')) == NULL );
  if( new != NULL ) *new = '\0';
 
  return ptr;
} 

static char
*chomp_line( FILE *fp, char **mystr, int *line_num )
{
  char *ptr;
  while( TRUE ){
    if(( *mystr = get_line( fp )) == NULL) return( NULL );
    (*line_num)++;
    ptr = trim( *mystr );
    /* exclude comments */
    if( *ptr != '#' && *ptr != '\0' ){
      return( ptr );
    }
  }
} 

int
load_conf( char *filename )
{
  FILE *fp;
  int  fd;
  char temp[32]; 
  int  line_num = 0;
  char *line;
  char *option;
  char *value;
 
  if (( fp = fopen(filename, "r")) == NULL ) {
    return -1;
  } 

  while(( line = chomp_line( fp, &line, &line_num )) != NULL ){
    option = line;
    while( *line && !ISSPACE( (int)*line ) && !ISSEPARATOR( *line ))
      line++;
    *line++=0;
    while( ISSPACE( (int)*line ) || ISSEPARATOR( *line ))
      line++;
    value  = line;
    while( *line && !ISCOMMENT(*line))
      line++;  
    *line++=0;
    trim( option );
    trim( value  );
    if( !strncasecmp( option, "hostname", 8   )){
      my.hostname = value;
    }
    if( !strncasecmp( option, "peername", 8   )){
      char *tmp = value;
      while( *tmp != '\0' && *tmp != ' ' ){ 
        *tmp = TOLOWER( *tmp ); 
        tmp++; 
      }
      *tmp++='\0';
      add_node(value);
    }
    if( !strncasecmp( option, "speed", 5   )){ 
      my.speed = atoi( value );
    }
    if( !strncasecmp( option, "timeout", 7 )){
      my.timeout = atoi( value );
    }
    if( !strncasecmp( option, "port", 4   )){
      my.portname = value;
    } 
  } /* end of while line=chomp_line */

  return 0;
}
