#include <lights.h>
#include <pattern.h>
#include <joedog/joedog.h>
#include <sys/kd.h>
#include <setup.h>

/**
 * local prototypes
 */
int  word_count( char pattern, char *s );
int  verify_pattern( char *str, int len );
wackyd_led_change_t get_pattern( char *pattern );

/**
 * blink
 * rotates the lights from num to caps to scroll
 * with a 10000 microsecond delay in succession.
 */ 
int
blink( int console, SPEED microseconds )
{
  wackyd_led_change_t num;
  wackyd_led_change_t caps;
  wackyd_led_change_t scr;
  wackyd_led_change_t pattern[4];

  num.num = caps.cap = scr.scr = WACKYD_LED_TURN_ON;
  num.cap = num.scr = caps.num = caps.scr = scr.num = scr.cap = WACKYD_LED_TURN_OFF;
  num.sleep = caps.sleep = scr.sleep = microseconds;

  pattern[0] = num;
  pattern[1] = caps;
  pattern[2] = scr;
  pattern[3] = caps;
 
  for( ;my.done != TRUE; ){
    render_pattern(console, pattern, 4);
  }

  return 0;
}

int
custom_blink( int console, char **words, int len )
{
  int x;
  wackyd_led_change_t *pattern;
  pattern = (wackyd_led_change_t*)xmalloc( len * sizeof(wackyd_led_change_t));

  for( x = 0; x < len; x ++ ){
    pattern[x] = get_pattern( words[x] );
  }  
  for( ;my.done != TRUE; ){
    render_pattern(console, pattern, len );
  }
  split_free( words, len );   
  pthread_exit( );
  return 0;
}

wackyd_led_change_t 
get_pattern( char *pattern )
{
  char *tmp;
  int  speed;
  wackyd_led_change_t led;

  /* assign optional speed value to temp */ 
  tmp      = substring( pattern, 3, strlen( pattern )); 
  speed    = atoi( tmp );
  led.num  = (TOLOWER(*(pattern+0))=='x')?WACKYD_LED_TURN_ON:WACKYD_LED_TURN_OFF;
  led.cap  = (TOLOWER(*(pattern+1))=='x')?WACKYD_LED_TURN_ON:WACKYD_LED_TURN_OFF;
  led.scr  = (TOLOWER(*(pattern+2))=='x')?WACKYD_LED_TURN_ON:WACKYD_LED_TURN_OFF;
  led.sleep= (speed==0)?300000:speed*100000;
  free( tmp ); 
  return led;
}

/**
 * esoteric function designed to verify
 * the integrity of the function, pattern
 * must match: [xo][xo][xo]NUM*
 */
int
verify_pattern( char *str, int len )
{
  char *walk, *save;
  char *tmp;
  int x;
 
  walk = save = str;
  save[len] = '\0';
  for( x=0; x<3; x++ ){
    if( ISX(*(walk+x)));
    else if( ISO(*(walk+x)));
    else{ return -1; }
  }
  tmp = substring( walk, x, len );
  if( atoi(tmp) < 0 ){
    joe_error( "Incorrect pattern, should be [xo][xo][xo]NUM,...\n" );
    free( tmp );
    return -1;
  }
  free( tmp );
  return 1;
}


